#Module for general helpers

import re, random, string
from unicodedata import normalize

def keygen(size=16, chars=string.letters+string.digits+'!@#$()_+-='):
    '''generates random key given a size'''
    return ''.join([ random.choice(chars) for i in xrange(size) ])

def slugify(string):
    '''creates a slug from a string'''
    if isinstance(string, str):
        string = string.decode('u8')
    string = normalize('NFKD', unicode(string.lower())).encode('ascii', errors='ignore')
    return re.sub('\W', '', re.sub('\s', '_', string)).replace('_', '-')


#Module for sqlalchemy helpers

def m2m(db, table1, table2):
    '''Creates the intermediate table for many-to-many relationships in SQLAlchemy'''
    table1, table2 = table1.lower(), table2.lower()
    return db.Table('%s_%s' % (table1, table2),
        db.Column('%s_id' % table1, db.Integer, db.ForeignKey('%s.id' % table1)),
        db.Column('%s_id' % table2, db.Integer, db.ForeignKey('%s.id' % table2)),
    )


import os, importlib
from flask import Blueprint as BP


class Blueprint(BP):

    def __init__(self, *args, **kwargs):
        if not kwargs.get('template_folder'):
            kwargs['template_folder'] = 'templates'
        if not kwargs.get('static_folder'):
            kwargs['static_folder'] = 'static'
        super(Blueprint, self).__init__(*args, **kwargs)

    @classmethod
    def attach_all(cls, app):
        cls.__load_by_convention(app)
        cls.__load_by_configuration(app)

    @classmethod
    def __load_by_convention(cls, app):
        blueprints_folder = app.config.get('BLUEPRINTS_FOLDER', 'blueprints')
        blueprints_path = os.path.join(app.root_path, blueprints_folder)

        if os.path.exists(blueprints_path):
            for blueprint_folder in os.listdir(blueprints_path):
                blueprint_folder_abspath = os.path.join(blueprints_path, blueprint_folder)

                if os.path.isdir(blueprint_folder_abspath):
                    blueprint = '%s.%s.%s' % (app.name, blueprints_folder, blueprint_folder)

                    views = cls.__import('%s.views' % blueprint)
                    app.register_blueprint(views.bp)

    @classmethod
    def __load_by_configuration(cls, app):
        blueprints = app.config.get('BLUEPRINTS')

        if blueprints:
            for module_name in blueprints:
                if app.name not in module_name:
                    module_name = '%s.%s' % (app.name, module_name)

                blueprint = cls.__import(module_name)

                for item in dir(blueprint):
                    if not item.startswith('_'):
                        attr = getattr(blueprint, item)

                        if issubclass(attr.__class__, BP):
                            app.register_blueprint(attr)
                            break

    @classmethod
    def __import(cls, module):
        err = u'Error while importing %s' % module
        try:
            print('importing: %s' % module)
            return importlib.import_module(module)
        except ImportError as e:
            raise ImportError('%s -> %s' % (err, e))
        except Exception as e:
            raise Exception('%s -> %s' % (err, e))

    def __repr__(self):
        return u'< %s in %s >' % (unicode(self.name), __name__)





"""
Flask-Nytro
-------------

Nytro is an extension to help developer providing a set of useful tools
giving even more facility to development apps with Flask.
"""
from setuptools import setup


setup(
    name='Flask-Nytro',
    version='1.1',
    url='http://github.com/sacanix/Flask-Nytro',
    license='BSD',
    author='Tony Kamillo (Sacanix)',
    author_email='tonysacanix@gmail.com',
    description='''Nytro is an extension to help developer providing a set of useful tools
    giving even more facility to development apps with Flask.''',
    long_description=__doc__,
    # if you would be using a package instead use packages instead
    # of py_modules:
    #py_modules=[''],
    packages=['flask_nytro', 'flask_nytro.helpers'],
    #zip_safe=False,
    include_package_data=True,
    platforms='any',
    install_requires=[
        'Flask'
    ],
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]
)